package com.gic.spark.filter;

import com.gic.spark.datasource.entity.DataSourceHive;
import com.gic.spark.datasource.mysql.MysqlRddManager;
import com.gic.spark.entity.bean.TrdEcuBrandBean;
import com.gic.spark.entity.bean.TrdEcuChannelBean;
import com.gic.spark.entity.bean.TrdEcuSalesBeanBase;
import com.gic.spark.entity.request.AbstractFilterRequest;
import com.gic.spark.entity.request.TagConsumeDoubleRequest;
import com.gic.spark.util.ConstantUtil;
import com.google.common.collect.Lists;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.Optional;
import scala.Tuple2;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * @description:
 * @author: wangxk
 * @date: 2020/5/7
 */
public abstract class AbstractTagConsumFilter implements BaseTagFilter {
    DataSourceHive dataSourceHiveBrandLabel = new DataSourceHive(ConstantUtil.ADS_GIC_TRD_ECU_BRAND_LABEL_D);
    DataSourceHive dataSourceHiveSalesLabel = new DataSourceHive(ConstantUtil.ADS_GIC_TRD_ECU_SALES_LABEL_D);

    public static class ConsumeStatisticEntity {
        Long ecuId;
        List<TrdEcuBrandBean> brandBeanList;
        List<TrdEcuChannelBean> channelBeanList;
    }

    protected static List<TrdEcuSalesBeanBase> filterSalesBean(ConsumeStatisticEntity entity, AbstractFilterRequest consumeRequest) {
        List<TrdEcuSalesBeanBase> result = new ArrayList<>();

        switch (consumeRequest.statisticsType) {
            case MCUINFO:
                for (TrdEcuChannelBean bean : entity.channelBeanList) {
                    if (consumeRequest.getStatisticsValList().contains(String.valueOf(bean.getMbr_area_id()))) {
                        result.add(bean);
                    }
                }
                break;
            case CHANNEL:
                for (TrdEcuChannelBean bean : entity.channelBeanList) {
                    if (consumeRequest.getStatisticsValList().contains(String.valueOf(bean.getChannel_code()))) {
                        result.add(bean);
                    }
                }
                break;
            case COMMODITYBRAND:
                for (TrdEcuBrandBean bean : entity.brandBeanList) {
                    if (consumeRequest.getStatisticsValList().contains(String.valueOf(bean.getEnt_brand_id()))) {
                        result.add(bean);
                    }
                }
                break;
            default:
                break;
        }

        return result;
    }

    protected static JavaRDD<TrdEcuSalesBeanBase> statisticsTypeHandle(JavaRDD<Tuple2<TrdEcuSalesBeanBase, Optional<Iterable<TrdEcuBrandBean>>>> labelRDD, AbstractFilterRequest consumeRequest) {
        JavaRDD<TrdEcuSalesBeanBase> consumeRDD = labelRDD.mapPartitions(data -> {
            List<TrdEcuSalesBeanBase> result = new ArrayList();
            while (data.hasNext()) {
                Tuple2<TrdEcuSalesBeanBase, Optional<Iterable<TrdEcuBrandBean>>> tp2 = data.next();
                TrdEcuSalesBeanBase consumeBean = tp2._1();
                switch (consumeRequest.getStatisticsType()) {
                    case COMMODITYBRAND:
                        if (tp2._2().isPresent()) {
                            for (TrdEcuBrandBean brandLabelBean : tp2._2().get()) {
                                if (consumeRequest.getStatisticsValList().contains(String.valueOf(brandLabelBean.getEnt_brand_id()))) {
                                    result.add(brandLabelBean);
                                }
                            }
                        }
                        break;
                    //TODO: 渠道重做
                    case CHANNEL:
                        if (consumeRequest.getStatisticsValList().contains(String.valueOf(consumeBean.getStore_info_id()))) {
                            result.add(consumeBean);
                        }
                        break;
                    case MCUINFO:
                        if (consumeRequest.getStatisticsValList().contains(String.valueOf(consumeBean.getMbr_area_id()))) {
                            result.add(consumeBean);
                        }
                        break;
                    default:
                        break;
                }
            }
            return result.iterator();
        });
        return consumeRDD;
    }

    protected JavaRDD<ConsumeStatisticEntity> getConsumeEntity(Integer enterpriseId) {
        JavaRDD<TrdEcuChannelBean> channelRDD = MysqlRddManager.getPojoFromDataset(dataSourceHiveSalesLabel.getDatasetByEntId(enterpriseId), TrdEcuChannelBean.class).javaRDD();
        JavaRDD<TrdEcuBrandBean> brandRDD = MysqlRddManager.getPojoFromDataset(dataSourceHiveBrandLabel.getDatasetByEntId(enterpriseId), TrdEcuBrandBean.class).javaRDD();

        JavaRDD<ConsumeStatisticEntity> rdd = channelRDD.mapToPair(data -> Tuple2.apply(data.getEcu_id(), data))
                .groupByKey()
                .leftOuterJoin(brandRDD.mapToPair(data -> Tuple2.apply(data.getEcu_id(), data)).groupByKey())
                .map(data -> {
                    Long ecuId = data._1();
                    Optional<Iterable<TrdEcuBrandBean>> optional = data._2()._2();
                    ConsumeStatisticEntity entity = new ConsumeStatisticEntity();
                    entity.ecuId = ecuId;
                    entity.channelBeanList = Lists.newArrayList(data._2()._1());
                    entity.brandBeanList = optional.isPresent() ? Lists.newArrayList(optional.get()) : new ArrayList<>();
                    return entity;
                });

        return rdd;
    }

    protected static TrdEcuSalesBeanBase brandLabelToSalesLabel(TrdEcuBrandBean brandLabelBean) {
        TrdEcuSalesBeanBase salesLabelBean = new TrdEcuSalesBeanBase();
        salesLabelBean.setEnt_id(brandLabelBean.getEnt_id());
        salesLabelBean.setMbr_area_id(brandLabelBean.getMbr_area_id());
        salesLabelBean.setEcu_id(brandLabelBean.getEcu_id());
        salesLabelBean.setStore_info_id(brandLabelBean.getStore_info_id());
        salesLabelBean.setOrder_times(brandLabelBean.getOrder_times());
        salesLabelBean.setSeff_goods_num(brandLabelBean.getSeff_goods_num());
        salesLabelBean.setSeff_order_cnt(brandLabelBean.getSeff_order_cnt());
        salesLabelBean.setReceive_amt(brandLabelBean.getReceive_amt());
        salesLabelBean.setPay_amt(brandLabelBean.getPay_amt());
        salesLabelBean.setTotal_amt(brandLabelBean.getTotal_amt());
        salesLabelBean.setSleep_days(brandLabelBean.getSleep_days());
        return salesLabelBean;
    }

    protected static void handleValueCompare(Set<String> groupIds, TagConsumeDoubleRequest consumeRequest, String tagGroupId, double cusSinglePiece) {
        switch (consumeRequest.getNumberType()) {
            case gt:
                if (cusSinglePiece > consumeRequest.getBeginNum()) {
                    groupIds.add(tagGroupId);
                }
                break;
            case gte:
                if (cusSinglePiece >= consumeRequest.getBeginNum()) {
                    groupIds.add(tagGroupId);
                }
                break;
            case lt:
                if (cusSinglePiece < consumeRequest.getEndNum()) {
                    groupIds.add(tagGroupId);
                }
                break;
            case lte:
                if (cusSinglePiece <= consumeRequest.getEndNum()) {
                    groupIds.add(tagGroupId);
                }
                break;
            case eq:
                if (cusSinglePiece == consumeRequest.getEqualNum()) {
                    groupIds.add(tagGroupId);
                }
                break;
            case between:
                if (cusSinglePiece >= consumeRequest.getBeginNum()
                        && cusSinglePiece <= consumeRequest.getEndNum()) {
                    groupIds.add(tagGroupId);
                }
            default:
                break;
        }
    }
}
