package com.gic.spark.filter;

import com.gic.spark.datasource.entity.DataSourceEntity;
import com.gic.spark.datasource.mysql.MysqlRddManager;
import com.gic.spark.entity.FilterProcessEntity;
import com.gic.spark.entity.bean.TrdEcuBrandBean;
import com.gic.spark.entity.bean.TrdEcuSalesBeanBase;
import com.gic.spark.entity.request.AbstractFilterRequest;
import com.gic.spark.entity.request.TagConsumeDoubleRequest;
import com.gic.spark.util.CommonUtil;
import com.google.common.base.Joiner;
import org.apache.spark.api.java.JavaPairRDD;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.Optional;
import scala.Tuple2;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @description: 连带率
 * @author: wangxk
 * @date: 2020/5/7
 */
public class TagAssociatedPurchaseRateFilter extends AbstractTagConsumFilter {

    private static TagAssociatedPurchaseRateFilter instance;

    public static TagAssociatedPurchaseRateFilter getInstance() {
        if (null == instance) {
            instance = new TagAssociatedPurchaseRateFilter();
        }
        return instance;
    }

    private TagAssociatedPurchaseRateFilter() {
    }

    @Override
    public List<DataSourceEntity> necessarySourceList() {
        List<DataSourceEntity> result = new ArrayList();
        result.add(dataSourceHiveBrandLabel);
        result.add(dataSourceHiveSalesLabel);
        return result;
    }

    @Override
    public JavaRDD<Long> filterValidMember(Integer enterpriseId, AbstractFilterRequest request) {
        TagConsumeDoubleRequest consumeRequest = (TagConsumeDoubleRequest) request;
        JavaRDD<TrdEcuSalesBeanBase> salesLabelRDD = MysqlRddManager.getPojoFromDataset(dataSourceHiveSalesLabel.getDatasetByEntId(enterpriseId), TrdEcuSalesBeanBase.class).javaRDD();
        JavaRDD<TrdEcuBrandBean> brandLabelRDD = MysqlRddManager.getPojoFromDataset(dataSourceHiveBrandLabel.getDatasetByEntId(enterpriseId), TrdEcuBrandBean.class).javaRDD();

        JavaRDD<Tuple2<TrdEcuSalesBeanBase, Optional<Iterable<TrdEcuBrandBean>>>> labelRDD = salesLabelRDD.mapToPair(data -> Tuple2.apply(data.getEcu_id(), data))
                .leftOuterJoin(brandLabelRDD.mapToPair(data -> Tuple2.apply(data.getEcu_id(), data)).groupByKey())
                .map(data -> data._2());

        JavaRDD<TrdEcuSalesBeanBase> consumeRDD = statisticsTypeHandle(labelRDD, consumeRequest);


        JavaRDD<Long> ecuRdd = consumeRDD.mapToPair(data -> Tuple2.apply(data.getEcu_id(), data))
                .reduceByKey((x, y) -> {
                    x.setSeff_goods_num(x.getSeff_goods_num() + y.getSeff_goods_num());
                    x.setSeff_order_cnt(x.getSeff_order_cnt() + y.getSeff_order_cnt());
                    return x;
                })
                .mapPartitions(data -> {
                    List<Long> result = new ArrayList();
                    while (data.hasNext()) {
                        Tuple2<Long, TrdEcuSalesBeanBase> tp2 = data.next();
                        double jointRate = CommonUtil.isEmptyInteger2int(tp2._2().getSeff_goods_num()) / CommonUtil.isEmptyInteger2int(tp2._2().getSeff_order_cnt());
                        switch (consumeRequest.getNumberType()) {
                            case gt:
                                if (jointRate > consumeRequest.getBeginNum()) {
                                    result.add(tp2._1());
                                }
                                break;
                            case gte:
                                if (jointRate >= consumeRequest.getBeginNum()) {
                                    result.add(tp2._1());
                                }
                                break;
                            case lt:
                                if (jointRate < consumeRequest.getEndNum()) {
                                    result.add(tp2._1());
                                }
                                break;
                            case lte:
                                if (jointRate <= consumeRequest.getEndNum()) {
                                    result.add(tp2._1());
                                }
                                break;
                            case eq:
                                if (jointRate == consumeRequest.getEqualNum()) {
                                    result.add(tp2._1());
                                }
                                break;
                            case between:
                                if (jointRate >= consumeRequest.getBeginNum()
                                        && jointRate <= consumeRequest.getEndNum()) {
                                    result.add(tp2._1());
                                }
                            default:
                                break;
                        }
                    }
                    return result.iterator();
                });
        return ecuRdd;
    }

    @Override
    public JavaPairRDD<Long, String> filterValidMember(Integer enterpriseId, List<FilterProcessEntity> processEntityList) {

        JavaRDD<ConsumeStatisticEntity> consumeEntity = getConsumeEntity(enterpriseId);

        JavaPairRDD<Long, String> resultRdd = consumeEntity.mapToPair(data -> {
            Set<String> groupIds = new HashSet<>();
            for (FilterProcessEntity entity : processEntityList) {
                TagConsumeDoubleRequest consumeRequest = (TagConsumeDoubleRequest) entity.getRequest();
                //过滤数据
                List<TrdEcuSalesBeanBase> salseBeanList = filterSalesBean(data, consumeRequest);

                if (salseBeanList.size() == 0) {
                    continue;
                }

                int goodsNum = 0;
                int orderCount = 0;
                for (TrdEcuSalesBeanBase beanBase : salseBeanList) {
                    goodsNum += beanBase.getSeff_goods_num();
                    orderCount += beanBase.getSeff_order_cnt();
                }

                double jointRate = goodsNum / (double) orderCount;
                String tagGroupId = entity.getTagGroupId();
                handleValueCompare(groupIds, consumeRequest, tagGroupId, jointRate);
            }


            return Tuple2.apply(data.ecuId, groupIds.size() > 0 ? Joiner.on(" ").join(groupIds) : null);
        }).filter(data -> data._2() != null);

        return resultRdd;
    }
}
